#-*- coding: utf-8 -*-

# Copyright 2008-2012 Calculate Ltd. http://www.calculate-linux.org
#
#  Licensed under the Apache License, Version 2.0 (the "License");
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.

import os, gettext
import threading
import types
import sys
from cl_overriding import __findFileMO

#gettext.find = __findFileMO

class lang:
    """Multilanguage class

    Using:
    import sys
    from calculate.lib.cl_lang import lang

    # set engligh language
    tr = lang('en')
    # set autodetect language
    tr = lang()

    # set translate domain calc
    tr.setGlobalDomain('calc')
    # set local translate domain
    #tr.setLocalDomain('calc')

    # set translate for this module
    tr.setLanguage(sys.modules[__name__])

    or
    lang(_local="calc",__setLang=sys.modules[__name__])
    """
    _modnames = {}
    GP = [""]

    def __init__(self):
        self.nameDomain = self.GP[0]
        #self.nameDomain = ''
        self.__catalog = None
        # translate language for all modules
        self._translators = {}

    def __translate(self,message):
        """Method return message without changes"""
        return self.__gettranslate()(message)

    def setLanguage(self,module,glob=False):
        """Set translate language for modules 'module'.

            module - export python module
            if the module export other modules, then lang will be
            set for them
            Method must be call after module for translate"""
        t = vars(module)
        if glob:
            for name,mod in vars(module).items():
                if type(mod) == types.ModuleType and \
                   not name.startswith('__') and \
                   not name in sys.builtin_module_names and \
                   (hasattr(mod,'__file__') and (
                   "calculate" in mod.__file__ or 
                   not mod.__file__.startswith('/usr/lib'))):
                    self.__setLang(mod)
                    self.setLanguage(mod,True)
        return self.__setLang(module)

    def __setLang(self,module):
        """ Set translate language for module 'module'.
        """
        if module.__name__ in self._modnames.keys():
            return True

        module._ = self.__translate
        self._modnames[module.__name__] = module._

    def __gettranslate(self):
        env = os.environ
        curThread = threading.currentThread()
        if hasattr(curThread,"lang"):
            l = curThread.lang
        elif env.has_key('LANG'):
            l = env['LANG'].split('.')[0].split("_")[0]
        else:
            l = "en"
        if l in self._translators:
            return self._translators[l]
        if l == 'en':
            trans = lambda x:x
        else:
            la = [l]
            reload(gettext)
            if gettext.find(self.nameDomain,self.__catalog,la):
                """Если найден словарь то инициализируем переводчик"""
                transl = gettext.translation(self.nameDomain,self.__catalog,la)
                trans = transl.gettext
            else:
                trans = lambda x:x
        self._translators[l] = trans
        return trans

    def getTranslatorByName(self,namemodule):
        """Method for detect already imported translate modules
        """
        if self._modnames.has_key(namemodule):
            return self._modnames[namemodule]
        return 0

    def setGlobalDomain(self, nameDomain):
        """ Method for set global translate domain
        """
        self.GP[0] = nameDomain
        self.nameDomain = self.GP[0]
        return True

    def setLocalDomain(self, nameDomain):
        """ Method for set local translate domain
        """
        self.nameDomain = nameDomain
        return True
 
def setGlobalTranslate(domain,*modules):
    _lang = lang()
    _lang.setGlobalDomain(domain)
    for mod in modules:
        _lang.setLanguage(mod,glob=True)

def setLocalTranslate(domain,*modules):
    _lang = lang()
    _lang.setLocalDomain(domain)
    for mod in modules:
        _lang.setLanguage(mod)

def getLazyLocalTranslate(translateFunc):
    class translate:
        def __init__(self,s):
            self.s = s
        def __str__(self):
            return translateFunc(self.s)
        def __hash__(self):
            return hash(self.s)
    return translate
